package main

import (
	"context"
	"fmt"
	"github.com/go-redis/redis/v8"
	consulapi "github.com/hashicorp/consul/api"
	"google.golang.org/grpc"
	"google.golang.org/grpc/keepalive"
	"google.golang.org/grpc/resolver"
	"google.golang.org/grpc/resolver/manual"
	"hilo-algoCenter/algo"
	"hilo-algoCenter/common/consul"
	"hilo-algoCenter/common/mylogrus"
	"hilo-algoCenter/protocol/userCenter"
	"time"
)

const (
	default_redis_address  = "47.244.34.27:6379"
	default_redis_password = "8QZ9JD1zLvPR3yHf"
	redis_section          = 0
)

var kacp = keepalive.ClientParameters{
	Time:                10 * time.Second, // send pings every 10 seconds if there is no activity
	Timeout:             time.Second,      // wait 1 second for ping ack before considering the connection dead
	PermitWithoutStream: true,             // send pings even without active streams
}

func main() {
	client, err := consulapi.NewClient(consulapi.DefaultConfig()) //非默认情况下需要设置实际的参数
	if err != nil {
		mylogrus.MyLog.Fatalln(err)
	}
	if client == nil {
		mylogrus.MyLog.Fatalln("Fail to get consul client.")
	}

	redisAddress := default_redis_address
	redisPassword := default_redis_password

	kv := client.KV()
	if kv != nil {
		p, _, err := kv.Get("redis_address", nil)
		if err == nil && p != nil {
			redisAddress = string(p.Value)
		}
		p, _, err = kv.Get("redis_password", nil)
		if err == nil && p != nil {
			redisPassword = string(p.Value)
		}
	}
	rdb := redis.NewClient(&redis.Options{
		Addr:     redisAddress,
		Password: redisPassword,
		DB:       redis_section,
	})
	if rdb == nil {
		mylogrus.MyLog.Fatalf("failed to connect redis %s\n", redisAddress)
	}

	result, err := rdb.Ping(context.Background()).Result()
	if err != nil {
		mylogrus.MyLog.Fatal(err)
	} else if result != "PONG" {
		mylogrus.MyLog.Fatalf("Invalid ping response %s", result)
	}

	cataLog := client.Catalog()
	if cataLog == nil {
		mylogrus.MyLog.Fatalln("No catalog.")
	}
	addr, err := consul.GetServices(cataLog, "userCenter")
	if err != nil {
		mylogrus.MyLog.Fatalln(err)
	}
	if len(addr) == 0 {
		mylogrus.MyLog.Fatalln("No userCenter available.")
	}

	addresses := make([]resolver.Address, len(addr))
	for i, s := range addr {
		addresses[i].Addr = s
		mylogrus.MyLog.Infof("address : %s", s)
	}

	r := manual.NewBuilderWithScheme("hilo")
	r.InitialState(resolver.State{Addresses: addresses})

	userCenterAddr := fmt.Sprintf("%s:///usercenter", r.Scheme())

	// Set up addresses connection to the userCenter.
	conn, err := grpc.Dial(userCenterAddr,
		grpc.WithInsecure(),
		grpc.WithBlock(),
		grpc.WithKeepaliveParams(kacp),
		grpc.WithResolvers(r),
		grpc.WithDefaultServiceConfig("{ \"loadBalancingPolicy\": \"round_robin\" }"))

	if err != nil {
		mylogrus.MyLog.Fatalf("did not connect: %v", err)
	}
	//defer conn.Close()
	userClient := userCenter.NewUserClient(conn)
	if userClient == nil {
		mylogrus.MyLog.Fatalln("userClient null")
	}
	algo.Start(userClient, rdb)

	select {}
}
